/**
 * Copyright (C) 2010-2013 Axel Morgner, structr <structr@structr.org>
 *
 * This file is part of structr <http://structr.org>.
 *
 * structr is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * structr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with structr.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.structr.core.predicate;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.neo4j.graphdb.Node;
import org.structr.common.SecurityContext;
import org.structr.core.Predicate;
import org.structr.core.entity.AbstractNode;

/**
 * A node predicate that evaluates to <b>true</b> if the type of the given node
 * matches the given type.
 *
 * @author Christian Morgner
 */
public class TypePredicate implements Predicate<Node> {

	private static final Logger logger = Logger.getLogger(TypePredicate.class.getName());
	private String type = null;

	public TypePredicate(String type) {
		this.type = type;
	}

	@Override
	public boolean evaluate(SecurityContext securityContext, Node... nodes) {
		
		if(nodes.length > 0) {

			Node node = nodes[0];
			
			if(node.hasProperty(AbstractNode.type.dbName())) {

				String value = (String)node.getProperty(AbstractNode.type.dbName());

				logger.log(Level.FINEST, "Type property: {0}, expected {1}", new Object[] { value, type } );

				return type.equals(value);

			} else {

				logger.log(Level.WARNING, "Node has no type property.");
			}
		}

		return false;
	}

}
